// reiserfs.cpp
//
// Copyright (c) 2003, Ingo Weinhold (bonefish@cs.tu-berlin.de)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// You can alternatively use *this file* under the terms of the the MIT
// license included in this package.

#include "Debug.h"
#include "SuperBlock.h"
#include "Volume.h"

// Information from Be Newsletter, Volume II, Issue 23, "Getting Mounted".

struct partition_data {
	char	partition_name[B_FILE_NAME_LENGTH];
	char	partition_type[B_FILE_NAME_LENGTH];
	char	file_system_short_name[B_FILE_NAME_LENGTH];
	char	file_system_long_name[B_FILE_NAME_LENGTH];
	char	volume_name[B_FILE_NAME_LENGTH];
	char	mounted_at[B_FILE_NAME_LENGTH];
	uint32	logical_block_size;
	uint64	offset;		/* in logical blocks from start of session */
	uint64	blocks;		/* in logical blocks */
	bool	hidden;		/* non-file system partition*/
	bool	reserved1;
	uint32	reserved2;
};

extern "C" bool ds_fs_id(partition_data*, int32, uint64, int32);

// ReiserFS doesn't feature volume names -- this one is used instead
static const char *kReiserFSDefaultVolumeName = REISERFS_DEFAULT_VOLUME_NAME;

// ds_fs_id
bool
ds_fs_id(partition_data *data, int32 device, uint64 sessionOffset,
		 int32 blockSize)
{
FUNCTION(("ds_fs_id(%p, %ld, %Lu, %ld)\n", data, device, offset, blockSize));
	bool result = false;
	SuperBlock superBlock;
	if (data) {
		off_t offset = (sessionOffset + data->offset) * blockSize;
		if (superBlock.Init(device, offset) == B_OK) {
			strcpy(data->file_system_short_name, "reiserfs");
			strcpy(data->file_system_long_name, "Reiser FS");
			strcpy(data->volume_name, kReiserFSDefaultVolumeName);
			result = true;
		}
	}
FUNCTION(("done: %d\n", result));
	return result;
}

