// read-seek-test.cpp

#include <algobase.h>
#include <stdio.h>
#include <string.h>

#include <Entry.h>
#include <File.h>
#include <Path.h>

// test_seek_backwards
void
test_read_backwards(BFile &file, uint8 *fileData, off_t fileSize,
					size_t readSize)
{
	uint8 *buffer = new uint8[readSize];
	int32 reads = (fileSize + readSize - 1) / readSize;
	status_t error = B_OK;
	for (int32 i = 0; error == B_OK && i < reads; i++) {
		off_t lastPosition = fileSize - i * readSize;
		off_t position = max(0LL, lastPosition - readSize);
		ssize_t toRead = lastPosition - position;
		if (file.Seek(position, SEEK_SET) >= 0) {
			ssize_t bytesRead = file.Read(buffer, toRead);
			if (bytesRead == toRead) {
				if (memcmp(buffer, fileData + position, bytesRead)) {
					error = B_ERROR;
					printf("backwards: buffers differ\n");
				}
			} else {
				error = B_IO_ERROR;
				printf("backwards: read failed: %ld bytes at %lld: %ld\n",
					   toRead, position, bytesRead);
			}
		} else {
			error = B_IO_ERROR;
			printf("backwards: seek failed\n");
		}
	}
	delete[] buffer;
}

// test_seek_forward
void
test_read_forward(BFile &file, uint8 *fileData, off_t fileSize,
				  size_t readSize)
{
	uint8 *buffer = new uint8[readSize];
	int32 reads = (fileSize + readSize - 1) / readSize;
	status_t error = B_OK;
	for (int32 i = 0; error == B_OK && i < reads; i++) {
		off_t position = i * readSize;
		ssize_t bytesRead = file.Read(buffer, readSize);
		if (bytesRead == min((off_t)readSize, fileSize - position)) {
			if (memcmp(buffer, fileData + position, bytesRead)) {
				error = B_ERROR;
				printf("forward: buffer differ\n");
			}
		} else {
			error = B_IO_ERROR;
			printf("forward: read failed\n");
		}
	}
	delete[] buffer;
}

// test_seek_read
void
test_seek_read(BFile &file, uint8 *fileData, off_t fileSize)
{
	size_t readSizes[] = { 128, 1024, 2073, 4096, 65536, 1024 * 1024 };
	int32 readSizesCount = sizeof(readSizes) / sizeof(size_t);
	for (int32 i = 0; i < readSizesCount; i++) {
		size_t readSize = readSizes[i];
		printf("read size: %lu\n", readSize);
		if (file.Seek(0, SEEK_SET) >= 0)
			test_read_forward(file, fileData, fileSize, readSize);
		else
			printf("seek to the beginning failed\n");
		if (file.Seek(0, SEEK_END) >= 0)
			test_read_backwards(file, fileData, fileSize, readSize);
		else
			printf("seek to the beginning failed\n");
	}
}

// main
int
main()
{
	const char *fileName = "/ReiserFS untitled/home/bonefish/pictures/"
						   "PortoAlegre/2002-12-18/pict0002.jpg";
	BFile file;
	off_t fileSize = 0;
	if (file.SetTo(fileName, B_READ_ONLY) == B_OK
		&& file.GetSize(&fileSize) == B_OK) {
		// read whole file into memory
		uint8 *fileData = new uint8[fileSize];
		if (file.Read(fileData, fileSize) == fileSize) {
			test_seek_read(file, fileData, fileSize);
		} else
			printf("failed to read file data\n");
		delete[] fileData;
	} else
		printf("failed to init file\n");
	return 0;
}

